<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 11/04/2009 *
 * ===================                          Last change: 11/04/2009 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : network_functions.php                            *
 * -------------------------------------------------------------------- *
 * Short description : Functions for ext-network                        *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Funktionen fuer ext-network                      *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2011 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Private setter for current network id
function setCurrentNetworkId ($networkId) {
	$GLOBALS['current_network_id'] = bigintval($networkId);
}

// Private getter for current network id
function getCurrentNetworkId () {
	return $GLOBALS['current_network_id'];
}

// Detects if a supported form has been sent
function detectNetworkProcessForm () {
	// 'do' must be provided in URL
	if (!isGetRequestElementSet('do')) {
		// Not provided!
		debug_report_bug(__FUNCTION__, __LINE__, 'No "do" has been provided. Please fix your templates.');
	} // END - if

	// Default is invalid
	$GLOBALS['network_form_name'] = 'invalid';

	// Now search all valid
	foreach (array('ok', 'edit', 'delete', 'do_edit', 'do_delete') as $form) {
		// Is it detected
		if (isFormSent($form)) {
			// Use this form name
			$GLOBALS['network_form_name'] = $form;

			// Abort loop
			break;
		} // END - if
	} // END - foreach

	// Has the form being detected?
	if ($GLOBALS['network_form_name'] == 'invalid') {
		// Not supported
		debug_report_bug(__FUNCTION__, __LINE__, 'POST form could not be detected.');
	} // END - if
}

// Handle a (maybe) sent form here
function doNetworkHandleForm () {
	// Do we have a form sent?
	if (countRequestPost() > 0) {
		// Detect sent POST form
		detectNetworkProcessForm();
	} elseif (!isGetRequestElementSet('do')) {
		// Skip any further requests
		return;
	}

	// Process the request
	doAdminNetworkProcessForm();
}

// Processes an admin form
function doAdminNetworkProcessForm () {
	// Create function name
	$functionName = sprintf("doAdminNetworkProcess%s", capitalizeUnderscoreString(getRequestElement('do')));

	// Is the function valid?
	if (!function_exists($functionName)) {
		// Invalid function name
		debug_report_bug(__FUNCTION__, __LINE__, 'Invalid do ' . getRequestElement('do') . ', function ' . $functionName .' does not exist.', false);
	} // END - if

	// Init global arrays
	$GLOBALS['network_types_disabled'] = array();

	// Call-back the method handling our request
	call_user_func($functionName);
}

// Checks wether the (short) network name is already used (valid)
function isNetworkNameValid ($name) {
	// Query for it
	$result = SQL_QUERY_ESC("SELECT `network_id` FROM `{?_MYSQL_PREFIX?}_network_data` WHERE `network_short_name`='%s' LIMIT 1",
		array($name), __FUNCTION__, __LINE__);

	// Does it exist?
	$isValid = (SQL_NUMROWS($result) == 1);

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $isValid;
}

// Checks wether the given network type is already used (valid)
function isNetworkTypeHandleValid ($type, $networkId) {
	// Query for it
	$result = SQL_QUERY_ESC("SELECT `network_type_id` FROM `{?_MYSQL_PREFIX?}_network_types` WHERE `network_id`=%s AND `network_type_handler`='%s' LIMIT 1",
		array(
			$networkId,
			$type
		), __FUNCTION__, __LINE__);

	// Does it exist?
	$isValid = (SQL_NUMROWS($result) == 1);

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $isValid;
}

// Checks wether the given network request parameter is already used (valid)
function isNetworkRequestElementValid ($key, $type, $networkId) {
	// Query for it
	$result = SQL_QUERY_ESC("SELECT `network_request_param_id` FROM `{?_MYSQL_PREFIX?}_network_request_params` WHERE `network_id`=%s AND `network_type_id`=%s AND `network_request_param_key`='%s' LIMIT 1",
		array(
			$networkId,
			$type,
			$key
		), __FUNCTION__, __LINE__);

	// Does it exist?
	$isValid = (SQL_NUMROWS($result) == 1);

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $isValid;
}

// Checks wether the given network API array translation
function isNetworkArrayTranslationValid ($key, $type, $networkId) {
	// Query for it
	$result = SQL_QUERY_ESC("SELECT `network_array_id` FROM `{?_MYSQL_PREFIX?}_network_array_translation` WHERE `network_id`=%s AND `network_type_id`=%s AND `network_array_index`='%s' LIMIT 1",
		array(
			$networkId,
			$type,
			$key
		), __FUNCTION__, __LINE__);

	// Does it exist?
	$isValid = (SQL_NUMROWS($result) == 1);

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $isValid;
}

// "Getter" for a network's data by provided id number
function getNetworkDataById ($networkId, $column = '') {
	// Ids lower one are not accepted
	if ($networkId < 1) {
		// Not good, should be fixed
		debug_report_bug(__FUNCTION__, __LINE__, 'Network id ' . $networkId . ' is smaller than 1.');
	} // END - if

	// Set current network id
	setCurrentNetworkId($networkId);

	// Is it cached?
	if (!isset($GLOBALS['network_data'][$networkId])) {
		// By default we have no data
		$GLOBALS['network_data'][$networkId] = array();

		// Query for the network data
		$result = SQL_QUERY_ESC("SELECT
	`network_id`,
	`network_short_name`,
	`network_title`,
	`network_reflink`,
	`network_data_separator`,
	`network_row_separator`,
	`network_request_type`,
	`network_charset`,
	`network_require_id_card`,
	`network_query_amount`
FROM
	`{?_MYSQL_PREFIX?}_network_data`
WHERE
	`network_id`=%s
LIMIT 1",
			array(bigintval($networkId)), __FUNCTION__, __LINE__);

		// Do we have an entry?
		if (SQL_NUMROWS($result) == 1) {
			// Then get it
			$GLOBALS['network_data'][$networkId] = SQL_FETCHARRAY($result);
		} // END - if

		// Free result
		SQL_FREERESULT($result);
	} // END - if

	// Return result
	if (empty($column)) {
		// Return array
		return $GLOBALS['network_data'][$networkId];
	} else {
		// Return column
		return $GLOBALS['network_data'][$networkId][$column];
	}
}

// "Getter" for a network's data by provided type id number
function getNetworkDataByTypeId ($networkId, $column = '') {
	// Ids lower one are not accepted
	if ($networkId < 1) {
		// Not good, should be fixed
		debug_report_bug(__FUNCTION__, __LINE__, 'Network type id ' . $networkId . ' is smaller than 1.');
	} // END - if

	// Set current network id
	setCurrentNetworkId($networkId);

	// Is it cached?
	if (!isset($GLOBALS['network_data'][$networkId])) {
		// By default we have no data
		$GLOBALS['network_data'][$networkId] = array();

		// Query for the network data
		$result = SQL_QUERY_ESC("SELECT
	d.`network_id`,
	d.`network_short_name`,
	d.`network_title`,
	d.`network_reflink`,
	d.`network_data_separator`,
	d.`network_row_separator`,
	d.`network_request_type`,
	d.`network_charset`,
	d.`network_require_id_card`,
	d.`network_query_amount`,
	t.`network_type_handler`,
	t.`network_type_api_url`,
	t.`network_type_click_url`,
	t.`network_type_banner_url`
FROM
	`{?_MYSQL_PREFIX?}_network_data` AS d
LEFT JOIN
	`{?_MYSQL_PREFIX?}_network_types` AS t
ON
	d.`network_id`=t.`network_id`
WHERE
	t.`network_type_id`=%s
LIMIT 1",
			array(bigintval($networkId)), __FUNCTION__, __LINE__);

		// Do we have an entry?
		if (SQL_NUMROWS($result) == 1) {
			// Then get it
			$GLOBALS['network_data'][$networkId] = SQL_FETCHARRAY($result);
		} // END - if

		// Free result
		SQL_FREERESULT($result);
	} // END - if

	// Return result
	if (empty($column)) {
		// Return array
		return $GLOBALS['network_data'][$networkId];
	} else {
		// Return column
		return $GLOBALS['network_data'][$networkId][$column];
	}
}

// "Getter" for a network type data by provided id number
function getNetworkTypeDataById ($networkId) {
	// Ids lower one are not accepted
	if ($networkId < 1) {
		// Not good, should be fixed
		debug_report_bug(__FUNCTION__, __LINE__, 'Network type id ' . $networkId . ' is smaller than 1.');
	} // END - if

	// By default we have no data
	$GLOBALS['network_type_data'][$networkId] = array();

	// Query for the network data
	$result = SQL_QUERY_ESC('SELECT
	`network_type_id`,
	`network_id`,
	`network_type_handler`,
	`network_type_api_url`,
	`network_type_click_url`,
	`network_type_banner_url`
FROM
	`{?_MYSQL_PREFIX?}_network_types`
WHERE
	`network_type_id`=%s
LIMIT 1',
		array(bigintval($networkId)), __FUNCTION__, __LINE__);

	// Do we have an entry?
	if (SQL_NUMROWS($result) == 1) {
		// Then get it
		$GLOBALS['network_type_data'][$networkId] = SQL_FETCHARRAY($result);
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $GLOBALS['network_type_data'][$networkId];
}

// "Getter" for a network request parameter data by provided id number
function getNetworkRequestParamsDataById ($networkId) {
	// Ids lower one are not accepted
	if ($networkId < 1) {
		// Not good, should be fixed
		debug_report_bug(__FUNCTION__, __LINE__, 'Network request parameter id ' . $networkId . ' is smaller than 1.');
	} // END - if

	// By default we have no data
	$networkRequestData = array();

	// Query for the network data
	$result = SQL_QUERY_ESC('SELECT
	`network_request_param_id`,
	`network_id`,
	`network_type_id`,
	`network_request_param_key`,
	`network_request_param_value`,
	`network_request_param_default`
FROM
	`{?_MYSQL_PREFIX?}_network_request_params`
WHERE
	`network_request_param_id`=%s
LIMIT 1',
		array(bigintval($networkId)), __FUNCTION__, __LINE__);

	// Do we have an entry?
	if (SQL_NUMROWS($result) == 1) {
		// Then get it
		$networkRequestData = SQL_FETCHARRAY($result);
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $networkRequestData;
}

// Updates given network (id) with data from array
function doNetworkUpdateDataByArray ($networkId, $networkData) {
	// Ids lower one are not accepted
	if ($networkId < 1) {
		// Not good, should be fixed
		debug_report_bug(__FUNCTION__, __LINE__, 'Network id ' . $networkId . ' is smaller than 1.');
	} // END - if

	// Just call our inner method
	return adminSaveSettings($networkData, '_network_data', sprintf("`network_id`=%s", bigintval($networkId)), array(), false, false);
}

// Updates given network type handler (id) with data from array
function doNetworkUpdateTypeByArray ($networkId, $networkTypeData) {
	// Ids lower one are not accepted
	if ($networkId < 1) {
		// Not good, should be fixed
		debug_report_bug(__FUNCTION__, __LINE__, 'Network type handler id ' . $networkId . ' is smaller than 1.');
	} // END - if

	// Just call our inner method
	return adminSaveSettings($networkTypeData, '_network_types', sprintf("`network_type_id`=%s", bigintval($networkId)), array(), false, false);
}

// Updates given network request parameters (id) with data from array
function doNetworkUpdateParamsByArray ($networkId, $networkParamData) {
	// Ids lower one are not accepted
	if ($networkId < 1) {
		// Not good, should be fixed
		debug_report_bug(__FUNCTION__, __LINE__, 'Network request parameter id ' . $networkId . ' is smaller than 1.');
	} // END - if

	// Just call our inner method
	return adminSaveSettings($networkParamData, '_network_request_params', sprintf("`network_request_param_id`=%s", bigintval($networkId)), array(), false, false);
}

// Removes given network entry
function doAdminRemoveNetworkEntry ($table, $column, $networkId, $limit = 1) {
	// Remove the entry
	SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM `{?_MYSQL_PREFIX?}_network_%s` WHERE `%s`=%s LIMIT %s",
		array(
			$table,
			$column,
			$networkId,
			$limit
		), __FUNCTION__, __LINE__);

	// Return affected rows
	return SQL_AFFECTEDROWS();
}

// Generates a list of networks for given script and returns it
function generateAdminNetworkList () {
	// Init output
	$content = '';

	// Query for all networks
	$result = SQL_QUERY("SELECT
	`network_id`,
	`network_short_name`,
	`network_title`
FROM
	`{?_MYSQL_PREFIX?}_network_data`
WHERE
	`network_active`='Y'
ORDER BY
	`network_short_name` ASC", __FUNCTION__, __LINE__);

	// Do we have entries?
	if (!SQL_HASZERONUMS($result)) {
		// List all entries
		$rows = array();
		while ($row = SQL_FETCHARRAY($result)) {
			// Is this valid, then add it
			if ((is_array($row)) && (isset($row['network_id']))) {
				// Add entry
				$rows[$row['network_id']] = $row;
			} // END - if
		} // END - while

		// Generate the selection box
		$content = generateSelectionBoxFromArray($rows, 'network_id', 'network_id', '', '', 'network');
	} else {
		// Nothing selected
		$content = loadTemplate('admin_settings_unsaved', false, '{--ADMIN_ENTRIES_404--}');
	}

	// Free the result
	SQL_FREERESULT($result);

	// Return the list
	return $content;
}

// Generator (somewhat getter) for a list of network types for given network id
function generateAdminNetworkTypeList ($networkId) {
	// Init content
	$content = '';

	// Query all types of this network
	$result = SQL_QUERY_ESC('SELECT
	`network_type_id`,
	`network_type_handler`
FROM
	`{?_MYSQL_PREFIX?}_network_types`
WHERE
	`network_id`=%s
ORDER BY
	`network_type_handler` ASC',
		array(
			bigintval($networkId)
		), __FUNCTION__, __LINE__);

	// Do we have entries?
	if (!SQL_HASZERONUMS($result)) {
		// List all entries
		$rows = array();
		while ($row = SQL_FETCHARRAY($result)) {
			// Is this valid, then add it
			if ((is_array($row)) && (isset($row['network_type_id']))) {
				// Add entry
				$rows[$row['network_type_id']] = $row;
			} // END - if
		} // END - while

		// Generate the selection box
		$content = generateSelectionBoxFromArray($rows, 'network_type', 'network_type_id');
	} else {
		// Nothing selected
		$content = loadTemplate('admin_settings_unsaved', false, '{--ADMIN_ENTRIES_404--}');
	}

	// Free the result
	SQL_FREERESULT($result);

	// Return content
	return $content;
}

// Generator (somewhat getter) for a list of network types for all types
function generateAdminDistinctNetworkTypeList () {
	// Init content
	$content = '';

	// Query all types of this network
	$result = SQL_QUERY("SELECT
	t.`network_type_id`,
	t.`network_type_handler`,
	d.`network_title`
FROM
	`{?_MYSQL_PREFIX?}_network_types` AS t
LEFT JOIN
	`{?_MYSQL_PREFIX?}_network_data` AS d
ON
	t.`network_id`=d.`network_id`
WHERE
	d.`network_active`='Y'
ORDER BY
	d.`network_short_name` ASC,
	t.`network_type_handler` ASC", __FUNCTION__, __LINE__);

	// Do we have entries?
	if (!SQL_HASZERONUMS($result)) {
		// List all entries
		$rows = array();
		while ($row = SQL_FETCHARRAY($result)) {
			// Is this valid, then add it
			if ((is_array($row)) && (isset($row['network_type_id']))) {
				// Add entry
				$rows[$row['network_type_id']] = $row;
			} // END - if
		} // END - while

		// Generate the selection box
		$content = generateSelectionBoxFromArray($rows, 'network_type', 'network_type_id', '', '_title');
	} else {
		// Nothing selected
		$content = loadTemplate('admin_settings_unsaved', false, '{--ADMIN_ENTRIES_404--}');
	}

	// Free the result
	SQL_FREERESULT($result);
	//* DEBUG: */ die('<pre>'.encodeEntities($content).'</pre>');

	// Return content
	return $content;
}

// Generator (somewhat getter) for network type options
function generateNetworkTypeOptions ($networkId) {
	// Is this an array, then we just came back from edit/delete actions
	if (is_array($networkId)) {
		$networkId = '';
	} // END - if

	// Is this cached?
	if (!isset($GLOBALS[__FUNCTION__][$networkId])) {
		// Generate output and cache it
		$GLOBALS[__FUNCTION__][$networkId] = generateOptionList(
			'network_types',
			'network_type_id',
			'network_type_handler',
			$networkId,
			'',
			sprintf(
				"WHERE `network_id`=%s",
				bigintval(getRequestElement('network_id'))
			),
			'',
			'translateNetworkTypeHandler'
		);
	} // END - if

	// Return content
	return $GLOBALS[__FUNCTION__][$networkId];
}

// Generates an options list of all available (hard-coded) handlers
function generateNetworkTypesAvailableOptions ($defaultType = NULL) {
	// Is it cached?
	if (!isset($GLOBALS[__FUNCTION__][$defaultType])) {
		// Generate list
		$GLOBALS[__FUNCTION__][$defaultType] = generateOptionList(
			'/ARRAY/',
			array(
				'banner',
				'banner_click',
				'banner_view',
				'button',
				'button_click',
				'button_view',
				'surfbar',
				'surfbar_click',
				'surfbar_view',
				'forcedbanner',
				'forcedtextlink',
				'textlink',
				'textlink_click',
				'textlink_view',
				'skybanner',
				'skybanner_click',
				'skybanner_view',
				'layer',
				'layer_click',
				'layer_view',
				'popup',
				'popdown',
				'textmail',
				'htmlmail',
				'lead',
				'sale',
				'payperactive',
				'pagepeel',
				'traffic'
			),
			array(),
			$defaultType,
			'', '',
			$GLOBALS['network_types_disabled'],
			'translateNetworkTypeHandler'
		);
	} // END - if

	// Return content
	return $GLOBALS[__FUNCTION__][$defaultType];
}

// Generates an options list (somewhat getter) ofr request keys
function generateNetworkRequestKeyOptions () {
	// Is it cached?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Generate and cache it
		$GLOBALS[__FUNCTION__] = generateOptionList(
			'/ARRAY/',
			array(
				'id',
				'sid',
				'hash',
				'password',
				'reload',
				'maximum_stay',
				'minimum_stay',
				'currency',
				'type',
				'remain',
				'reward',
				'size',
				'erotic',
				'extra',
				'country'
			),
			array(),
			'',
			'', '',
			$GLOBALS['network_request_params_disabled'],
			'translateNetworkRequestParamKey'
		);
	} // END - if

	// Return content
	return $GLOBALS[__FUNCTION__];
}

// Generator (somewhat getter) for (return) array translation
function generateNetworkTranslationOptions ($default = '') {
	// Is it cached?
	if (!isset($GLOBALS[__FUNCTION__][$default])) {
		// Generate and cache it
		$GLOBALS[__FUNCTION__][$default] = generateOptionList(
			'network_translations',
			'network_translation_id',
			'network_translation_name',
			$default,
			'',
			'',
			$GLOBALS['network_translation_disabled'],
			'translateNetworkTranslationName'
		);
	} // END - if

	// Return content
	return $GLOBALS[__FUNCTION__][$default];
}

// Generates an option list of request types
function generateNetworkRequestTypeOptions ($default = '') {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__][$default])) {
		// Generate the list
		$GLOBALS[__FUNCTION__][$default] = generateOptionList(
			'/ARRAY/',
			array(
				'GET',
				'POST'
			),
			array(
				'{--ADMIN_NETWORK_REQUEST_TYPE_GET--}',
				'{--ADMIN_NETWORK_REQUEST_TYPE_POST--}'
			),
			$default
		);
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$default];
}

// Generates an option list of network_api_active
function generateNetworkApiActiveOptions ($default = '') {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__][$default])) {
		// Generate the list
		$GLOBALS[__FUNCTION__][$default] = generateYesNoOptionList($default);
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$default];
}

// Translates 'translate_name' for e.g. templates
function translateNetworkTranslationName ($name) {
	// Generate id
	$messageId = 'ADMIN_NETWORK_TRANSLATE_' . strtoupper($name) . '_NAME';

	// Is the message id there?
	if (!isMessageIdValid($messageId)) {
		// Not valid type
		debug_report_bug(__FUNCTION__, __LINE__, 'type=' . $type . ' is invalid.');
	} // END - if

	// Return message id
	return '{--' . $messageId . '--}';
}

// Translates the network type handler (e.g. banner, paidmail) for templates
function translateNetworkTypeHandler ($type) {
	// Generate id
	$messageId = 'ADMIN_NETWORK_TYPE_HANDLER_' . strtoupper($type);

	// Is the message id there?
	if (!isMessageIdValid($messageId)) {
		// Not valid type
		debug_report_bug(__FUNCTION__, __LINE__, 'type=' . $type . ' is invalid.');
	} // END - if

	// Return message id
	return '{--' . $messageId . '--}';
}

// Translates request type
function translateNetworkRequestType ($type) {
	// Generate id
	$messageId = 'ADMIN_NETWORK_REQUEST_TYPE_' . strtoupper($type) . '';

	// Is the message id there?
	if (!isMessageIdValid($messageId)) {
		// Not valid type
		debug_report_bug(__FUNCTION__, __LINE__, 'type=' . $type . ' is invalid.');
	} // END - if

	// Return message id
	return '{--' . $messageId . '--}';
}

// Translates request parameter
function translateNetworkRequestParamKey ($param) {
	// Generate id
	$messageId = 'ADMIN_NETWORK_REQUEST_PARAMETER_' . strtoupper($param) . '';

	// Is the message id there?
	if (!isMessageIdValid($messageId)) {
		// Not valid param
		debug_report_bug(__FUNCTION__, __LINE__, 'param=' . $param . ' is invalid.');
	} // END - if

	// Return message id
	return '{--' . $messageId . '--}';
}

// Translates API index
function translateNetworkApiIndex ($index) {
	// Do we have cache?
	if (!isset($GLOBALS['network_array_index'])) {
		// Get an array of all API array indexes
		$GLOBALS['network_array_index'] = array();

		// Get all entries
		$result = SQL_QUERY('SELECT
	`network_array_id`,
	`network_array_index`,
	`network_translation_name`
FROM
	`{?_MYSQL_PREFIX?}_network_array_translation`
INNER JOIN
	`{?_MYSQL_PREFIX?}_network_translations`
ON
	`network_array_index`=`network_translation_id`
ORDER BY
	`sort` ASC', __FUNCTION__, __LINE__);

		// Do we have entries?
		if (!SQL_HASZERONUMS($result)) {
			// Get all entries
			while ($row = SQL_FETCHARRAY($result)) {
				// Add it to our global array
				$GLOBALS['network_array_index'][$row['network_array_index']] = $row;
			} // END - while
		} // END - if

		// Free result
		SQL_FREERESULT($result);
	} // END - if

	// Default name is unknown
	$name = 'unknown';

	// Is the entry there?
	if (isset($GLOBALS['network_array_index'][$index])) {
		// Then get the name
		$name = $GLOBALS['network_array_index'][$index]['network_translation_name'];
	} // END - if

	// Return translation
	return translateNetworkTranslationName($name);
}

// Translates network API configuration status (see function isNetworkApiConfigured()) by given id
function translateNetworkApiConfiguredStatusById ($networkId) {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__][$networkId])) {
		// By default it is not configured
		$GLOBALS[__FUNCTION__][$networkId] = '{--ADMIN_NETWORK_API_NOT_CONFIGURED--}';

		// So is it configured?
		if (isNetworkApiConfigured($networkId)) {
			// Yes, it is
			$GLOBALS[__FUNCTION__][$networkId] = '{--ADMIN_NETWORK_API_CONFIGURED--}';
		} // END - if
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$networkId];
}

// Checks if the given network is configured by looking its API configuration entry up
function isNetworkApiConfigured ($networkId) {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__][$networkId])) {
		// Check for an entry in network_api_config
		$GLOBALS[__FUNCTION__][$networkId] = (countSumTotalData(
			bigintval($networkId),
			'network_api_config',
			'network_id',
			'network_id',
			true
		) == 1);
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$networkId];
}

// Checks wether the given network type handler is configured
function isNetworkTypeHandlerConfigured ($networkId, $networkTypeId) {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__][$networkId][$networkTypeId])) {
		// Determine it
		$GLOBALS[__FUNCTION__][$networkId][$networkTypeId] = (countSumTotalData(
			bigintval($networkTypeId),
			'network_types_config',
			'network_data_id',
			'network_type_id',
			true,
			sprintf(' AND `network_id`=%s', bigintval($networkId))
		) == 1);
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$networkId][$networkTypeId];
}

//------------------------------------------------------------------------------
//                             Call-back functions
//------------------------------------------------------------------------------

// Callback function to add new network
function doAdminNetworkProcessAddNetwork () {
	// We can say here, the form is sent, so check if the network is already added
	if (isNetworkNameValid(postRequestElement('network_short_name'))) {
		// Already there
		loadTemplate('admin_settings_unsaved', false, '{%message,ADMIN_NETWORK_ALREADY_ADDED=' . postRequestElement('network_short_name') . '%}');
		return false;
	} // END - if

	// Remove the 'ok' part
	unsetPostRequestElement('ok');

	// Add the whole request to database
	SQL_QUERY(getInsertSqlFromArray(postRequestArray(), 'network_data'), __FUNCTION__, __LINE__);

	// Add the id for output only
	setPostRequestElement('network_id', SQL_INSERTID());

	// Output message
	if (!SQL_HASZEROAFFECTED()) {
		// Successfully added
		loadTemplate('admin_network_added', false, postRequestArray());
	} else {
		// Not added
		loadTemplate('admin_settings_unsaved', false, '{%message,ADMIN_NETWORK_DATA_NOT_ADDED=' . postRequestElement('network_short_name') . '%}');
	}
}

// Displays selected networks for editing
function doAdminNetworkProcessHandleNetworks () {
	// Do we have selections?
	if (ifPostContainsSelections()) {
		// Something has been selected, so start displaying one by one
		$OUT = '';
		foreach (postRequestElement('sel') as $networkId => $sel) {
			// Is this selected?
			if ($sel == 1) {
				// Load this network's data
				$networkData = getNetworkDataById($networkId);

				// Do we have found the network?
				if (count($networkData) > 0) {
					// Add row template with given form name
					$OUT .= loadTemplate('admin_' . $GLOBALS['network_form_name'] . '_networks_row', true, $networkData);
				} // END - if
			} // END - if
		} // END - foreach

		// If we have no rows, we don't need to display the edit form
		if (!empty($OUT)) {
			// Output main template
			loadTemplate('admin_' . $GLOBALS['network_form_name'] . '_networks', false, $OUT);

			// Don't display the list/add new form
			$GLOBALS['network_display'] = false;
		} else {
			// Nothing selected/found
			loadTemplate('admin_settings_unsaved', false, '{--ADMIN_NETWORK_NOTHING_FOUND--}');
		}
	} // END - if
}

// Handle network type form
function doAdminNetworkProcessHandleNetworkTypes () {
	// Do we have selections?
	if (ifPostContainsSelections()) {
		// Load network data
		$networkData = getNetworkDataById(getRequestElement('network_id'));

		// Something has been selected, so start displaying one by one
		$OUT = '';
		foreach (postRequestElement('sel') as $networkId => $sel) {
			// Is this selected?
			if ($sel == 1) {
				// Load this network's data
				$networkTypeData = getNetworkTypeDataById($networkId);

				// Do we have found the network?
				if (count($networkTypeData) > 0) {
					if (isFormSent('edit')) {
						// Add row template for deleting
						$OUT .= loadTemplate('admin_edit_network_types_row', true, $networkTypeData);
					} elseif (isFormSent('delete')) {
						// Add row template for deleting
						$OUT .= loadTemplate('admin_delete_network_types_row', true, $networkTypeData);
					} else {
						// Problem!
						debug_report_bug(__FUNCTION__, __LINE__, 'Cannot detect edit/del.');
					}
				} // END - if
			} // END - if
		} // END - foreach

		// If we have no rows, we don't need to display the edit form
		if (!empty($OUT)) {
			// Output main template
			if (isFormSent('edit')) {
				loadTemplate('admin_edit_network_types', false, $OUT);
			} elseif (isFormSent('delete')) {
				loadTemplate('admin_delete_network_types', false, $OUT);
			} else {
				// Problem!
				debug_report_bug(__FUNCTION__, __LINE__, 'Cannot detect edit/del.');
			}

			// Don't display the list/add new form
			$GLOBALS['network_display'] = false;
		} else {
			// Nothing selected/found
			loadTemplate('admin_settings_unsaved', false, '{--ADMIN_NETWORK_TYPE_HANDLER_NOTHING_FOUND--}');
		}
	} // END - if
}

// Handle network request parameter form
function doAdminNetworkProcessHandleRequestParams () {
	// Do we have selections?
	if (ifPostContainsSelections()) {
		// Init cache array
		$GLOBALS['network_request_params_disabled'] = array();

		// Load network data
		$networkData = getNetworkDataById(getRequestElement('network_id'));

		// Something has been selected, so start displaying one by one
		$OUT = '';
		foreach (postRequestElement('sel') as $networkId => $sel) {
			// Is this selected?
			if ($sel == 1) {
				// Load this network's data
				$networkRequestData = getNetworkRequestParamsDataById($networkId);

				// Do we have found the network?
				if (count($networkRequestData) > 0) {
					if (isFormSent('edit')) {
						// Add row template for deleting
						$OUT .= loadTemplate('admin_edit_network_request_params_row', true, $networkRequestData);
					} elseif (isFormSent('delete')) {
						// Get type data
						$networkRequestData['network_type_data'] = getNetworkTypeDataById($networkRequestData['network_type_id']);

						// Add row template for deleting
						$OUT .= loadTemplate('admin_delete_network_request_params_row', true, $networkRequestData);
					} else {
						// Problem!
						debug_report_bug(__FUNCTION__, __LINE__, 'Cannot detect edit/del.');
					}
				} // END - if
			} // END - if
		} // END - foreach

		// If we have no rows, we don't need to display the edit form
		if (!empty($OUT)) {
			// Output main template
			if (isFormSent('edit')) {
				loadTemplate('admin_edit_network_request_params', false, $OUT);
			} elseif (isFormSent('delete')) {
				loadTemplate('admin_delete_network_request_params', false, $OUT);
			} else {
				// Problem!
				debug_report_bug(__FUNCTION__, __LINE__, 'Cannot detect edit/del.');
			}

			// Don't display the list/add new form
			$GLOBALS['network_display'] = false;
		} else {
			// Nothing selected/found
			loadTemplate('admin_settings_unsaved', false, '{--ADMIN_NETWORK_REQUEST_PARAMETER_NOTHING_FOUND--}');
		}
	} // END - if
}

// Changes given networks
function doAdminNetworkProcessChangeNetworks () {
	// Do we have selections?
	if (ifPostContainsSelections()) {
		// By default nothing is updated
		$updated = 0;

		// Something has been selected, so start updating them
		foreach (postRequestElement('sel') as $networkId => $sel) {
			// Update this entry?
			if ($sel == 1) {
				// Init data array
				$networkData = array();

				// Transfer whole array, except 'sel'
				foreach (postRequestArray() as $key => $entry) {
					// Skip 'sel' and submit button
					if (in_array($key, array('sel', 'do_edit'))) {
						continue;
					} // END - if

					// Do we have this enty?
					if (!isset($entry[$networkId])) {
						// Not found, needs fixing
						debug_report_bug(__FUNCTION__, __LINE__, 'No entry in key=' . $key . ', id=' . $networkId . ' found.');
					} // END - if

					// Add this entry
					$networkData[$key] = $entry[$networkId];
				} // END - foreach

				// Update the network data
				$updated += doNetworkUpdateDataByArray($networkId, $networkData);
			} // END - if
		} // END - foreach

		// Do we have updates?
		if ($updated > 0) {
			// Updates done
			displayMessage('{%message,ADMIN_NETWORK_UPDATED=' . $updated . '%}');
		} else {
			// Nothing changed
			loadTemplate('admin_settings_unsaved', false, '{--ADMIN_NETWORK_NOTHING_CHANGED--}');
		}
	} // END - if
}

// Removes given networks
function doAdminNetworkProcessRemoveNetworks () {
	// Do we have selections?
	if (ifPostContainsSelections()) {
		// By default nothing is removed
		$removed = 0;

		// Something has been selected, so start updating them
		foreach (postRequestElement('sel') as $networkId => $sel) {
			// Update this entry?
			if ($sel == 1) {
				// Remove this entry
				$removed += doAdminRemoveNetworkEntry('data', 'network_id', $networkId);
			} // END - if
		} // END - foreach

		// Do we have removes?
		if ($removed > 0) {
			// Removals done
			displayMessage('{%message,ADMIN_NETWORK_REMOVED=' . $removed . '%}');
		} else {
			// Nothing removed
			loadTemplate('admin_settings_unsaved', false, '{--ADMIN_NETWORK_NOTHING_REMOVED--}');
		}
	} // END - if
}

// Add a network type handler if not yet found
function doAdminNetworkProcessAddNetworkType () {
	// Is the network type handle already used with given network?
	if (isNetworkTypeHandleValid(postRequestElement('network_type_handler'), getRequestElement('network_id'))) {
		// Already added
		loadTemplate('admin_settings_unsaved', false, '{%message,ADMIN_NETWORK_TYPE_HANDLER_ALREADY_ADDED=' . postRequestElement('network_type_handler') . '%}');

		// ... so abort here
		return false;
	} // END - if

	// Remove the 'ok' part
	unsetPostRequestElement('ok');

	// Add id
	setPostRequestElement('network_id', bigintval(getRequestElement('network_id')));

	// Is network_type_banner_url set?
	if (postRequestElement('network_type_banner_url') == '') {
		// Remove empty value to get a NULL for an optional entry
		unsetPostRequestElement('network_type_banner_url');
	} // END - if

	// Add the whole request to database
	SQL_QUERY(getInsertSqlFromArray(postRequestArray(), 'network_types'), __FUNCTION__, __LINE__);

	// Output message
	if (!SQL_HASZEROAFFECTED()) {
		// Successfully added
		loadTemplate('admin_network_type_added', false, postRequestArray());
	} else {
		// Not added
		loadTemplate('admin_settings_unsaved', false, '{%message,ADMIN_NETWORK_TYPE_HANDLER_NOT_ADDED=' . postRequestElement('network_type_handler') . '%}');
	}
}

// Changes given network type handlers
function doAdminNetworkProcessChangeHandlerTypes () {
	// Do we have selections?
	if (ifPostContainsSelections()) {
		// By default nothing is updated
		$updated = 0;

		// Something has been selected, so start updating them
		foreach (postRequestElement('sel') as $networkId => $sel) {
			// Update this entry?
			if ($sel == 1) {
				// Init data array
				$networkTypeData = array();

				// Transfer whole array, except 'sel'
				foreach (postRequestArray() as $key => $entry) {
					// Skip 'sel' and submit button
					if (in_array($key, array('sel', 'do_edit'))) {
						continue;
					} // END - if

					// Do we have this enty?
					if (!isset($entry[$networkId])) {
						// Not found, needs fixing
						debug_report_bug(__FUNCTION__, __LINE__, 'No entry in key=' . $key . ', id=' . $networkId . ' found.');
					} // END - if

					// Fix empty network_type_banner_url to NULL
					if (($key == 'network_type_banner_url') && (trim($entry[$networkId]) == '')) {
						// Set it to NULL
						$entry[$networkId] = NULL;
					} // END - if

					// Add this entry
					$networkTypeData[$key] = $entry[$networkId];
				} // END - foreach

				// Update the network data
				$updated += doNetworkUpdateTypeByArray($networkId, $networkTypeData);
			} // END - if
		} // END - foreach

		// Do we have updates?
		if ($updated > 0) {
			// Updates done
			displayMessage('{%message,ADMIN_NETWORK_TYPE_HANDLER_UPDATED=' . $updated . '%}');
		} else {
			// Nothing changed
			loadTemplate('admin_settings_unsaved', false, '{--ADMIN_NETWORK_TYPE_HANDLER_NOTHING_CHANGED--}');
		}
	} // END - if
}

// Changes given network request parameters
function doAdminNetworkProcessChangeRequestParams () {
	// Do we have selections?
	if (ifPostContainsSelections()) {
		// By default nothing is updated
		$updated = 0;

		// Something has been selected, so start updating them
		foreach (postRequestElement('sel') as $networkId => $sel) {
			// Update this entry?
			if ($sel == 1) {
				// Init data array
				$networkParamsData = array();

				// Transfer whole array, except 'sel'
				foreach (postRequestArray() as $key => $entry) {
					// Skip 'sel' and submit button
					if (in_array($key, array('sel', 'do_edit'))) {
						continue;
					} // END - if

					// Do we have this enty?
					if (!isset($entry[$networkId])) {
						// Not found, needs fixing
						debug_report_bug(__FUNCTION__, __LINE__, 'No entry in key=' . $key . ', id=' . $networkId . ' found.');
					} // END - if

					// Fix empty network_request_param_default to NULL
					if (($key == 'network_request_param_default') && (trim($entry[$networkId]) == '')) {
						// Set it to NULL
						$entry[$networkId] = NULL;
					} // END - if

					// Add this entry
					$networkParamsData[$key] = $entry[$networkId];
				} // END - foreach

				// Update the network data
				$updated += doNetworkUpdateParamsByArray($networkId, $networkParamsData);
			} // END - if
		} // END - foreach

		// Do we have updates?
		if ($updated > 0) {
			// Updates done
			displayMessage('{%message,ADMIN_NETWORK_REQUEST_PARAMETER_UPDATED=' . $updated . '%}');
		} else {
			// Nothing changed
			loadTemplate('admin_settings_unsaved', false, '{--ADMIN_NETWORK_REQUEST_PARAMETER_NOTHING_CHANGED--}');
		}
	} // END - if
}

// Removes given network type handlers
function doAdminNetworkProcessRemoveNetworkTypes () {
	// Do we have selections?
	if (ifPostContainsSelections()) {
		// By default nothing is removed
		$removed = 0;

		// Something has been selected, so start updating them
		foreach (postRequestElement('sel') as $networkId => $sel) {
			// Update this entry?
			if ($sel == 1) {
				// Remove this entry
				$removed += doAdminRemoveNetworkEntry('types', 'network_type_id', $networkId);
			} // END - if
		} // END - foreach

		// Do we have removes?
		if ($removed > 0) {
			// Removals done
			displayMessage('{%message,ADMIN_NETWORK_TYPE_HANDLER_REMOVED=' . $removed . '%}');
		} else {
			// Nothing removed
			loadTemplate('admin_settings_unsaved', false, '{--ADMIN_NETWORK_TYPE_HANDLER_NOTHING_REMOVED--}');
		}
	} // END - if
}

// Removes given network request parameters
function doAdminNetworkProcessRemoveNetworkRequestParams () {
	// Do we have selections?
	if (ifPostContainsSelections()) {
		// By default nothing is removed
		$removed = 0;

		// Something has been selected, so start updating them
		foreach (postRequestElement('sel') as $networkId => $sel) {
			// Update this entry?
			if ($sel == 1) {
				// Remove this entry
				$removed += doAdminRemoveNetworkEntry('request_params', 'network_request_param_id', $networkId);
			} // END - if
		} // END - foreach

		// Do we have removes?
		if ($removed > 0) {
			// Removals done
			displayMessage('{%message,ADMIN_NETWORK_REQUEST_PARAMETER_REMOVED=' . $removed . '%}');
		} else {
			// Nothing removed
			loadTemplate('admin_settings_unsaved', false, '{--ADMIN_NETWORK_REQUEST_PARAMETER_NOTHING_REMOVED--}');
		}
	} // END - if
}

// Adds a request parameter to given network and type
function doAdminNetworkProcessAddRequestParam () {
	// Is the request parameter already used with given network?
	if (isNetworkRequestElementValid(postRequestElement('network_request_param_key'), postRequestElement('network_type_id'), getRequestElement('network_id'))) {
		// Already added
		loadTemplate('admin_settings_unsaved', false, '{%message,ADMIN_NETWORK_REQUEST_PARAMETER_ALREADY_ADDED=' . postRequestElement('network_request_param_key') . '%}');

		// ... so abort here
		return false;
	} // END - if

	// Remove the 'ok' part
	unsetPostRequestElement('ok');

	// Add id
	setPostRequestElement('network_id', bigintval(getRequestElement('network_id')));

	// Is network_request_param_default set?
	if (postRequestElement('network_request_param_default') == '') {
		// Remove empty value to get a NULL for an optional entry
		unsetPostRequestElement('network_request_param_default');
	} // END - if

	// Add the whole request to database
	SQL_QUERY(getInsertSqlFromArray(postRequestArray(), 'network_request_params'), __FUNCTION__, __LINE__);

	// Output message
	if (!SQL_HASZEROAFFECTED()) {
		// Successfully added
		loadTemplate('admin_network_request_param_added', false, postRequestArray());
	} else {
		// Not added
		loadTemplate('admin_settings_unsaved', false, '{%message,ADMIN_NETWORK_REQUEST_PARAMETER_NOT_ADDED=' . postRequestElement('network_request_param_key') . '%}');
	}
}

// Adds a API response array entry
function doAdminNetworkProcessAddNetworkArrayTranslation () {
	// Is the request parameter already used with given network?
	if (isNetworkArrayTranslationValid(postRequestElement('network_array_index'), postRequestElement('network_type_id'), getRequestElement('network_id'))) {
		// Already added
		loadTemplate('admin_settings_unsaved', false, '{%message,ADMIN_NETWORK_ARRAY_TRANSLATION_ALREADY_ADDED=' . postRequestElement('network_array_index') . '%}');

		// ... so abort here
		return false;
	} // END - if

	// Remove the 'ok' part
	unsetPostRequestElement('ok');

	// Add id
	setPostRequestElement('network_id', bigintval(getRequestElement('network_id')));

	// Add sorting
	setPostRequestElement('sort', (countSumTotalData(
		bigintval(postRequestElement('network_id')),
		'network_array_translation',
		'network_array_id',
		'network_id',
		true,
		sprintf(" AND `network_type_id`=%s", bigintval(postRequestElement('network_type_id')))
	) + 1));

	// Add the whole request to database
	SQL_QUERY(getInsertSqlFromArray(postRequestArray(), 'network_array_translation'), __FUNCTION__, __LINE__);

	// Output message
	if (!SQL_HASZEROAFFECTED()) {
		// Successfully added
		loadTemplate('admin_network_array_translation_added', false, postRequestArray());
	} else {
		// Not added
		loadTemplate('admin_settings_unsaved', false, '{%message,ADMIN_NETWORK_ARRAY_TRANSLATION_NOT_ADDED=' . postRequestElement('network_array_index') . '%}');
	}
}

// Adds/update network API configuration
function doAdminNetworkProcessNetworkApiConfig () {
	// Remove the 'ok' part
	unsetPostRequestElement('ok');

	// Add id
	setPostRequestElement('network_id', bigintval(getRequestElement('network_id')));

	// Is network_api_referral_button set?
	if (postRequestElement('network_api_referral_button') == '') {
		// Remove empty value to get a NULL for an optional entry
		unsetPostRequestElement('network_api_referral_button');
	} // END - if

	// Is there already an entry?
	if (isNetworkApiConfigured(getRequestElement('network_id'))) {
		// Generate SQL query
		$SQL = getUpdateSqlFromArray(postRequestArray(), 'network_api_config', 'network_id', postRequestElement('network_id'), array('network_id'));
	} else {
		// Insert new entry
		$SQL = getInsertSqlFromArray(postRequestArray(), 'network_api_config');
	}

	// Run the query
	SQL_QUERY($SQL, __FUNCTION__, __LINE__);

	// Output message
	if (!SQL_HASZEROAFFECTED()) {
		// Successfully added
		displayMessage('{--ADMIN_CONFIG_NETWORK_API_SAVED--}');
	} else {
		// Not added
		loadTemplate('admin_settings_unsaved', false, '{--ADMIN_CONFIG_NETWORK_API_NOT_SAVED--}');
	}
}

// Only adds network type configuration if not yet present
function doAdminNetworkProcessAddHandlerTypesConfig ($displayMessage = true) {
	// Remove the 'ok' part
	unsetPostRequestElement('ok');

	// Add both ids
	setPostRequestElement('network_id', bigintval(getRequestElement('network_id')));
	setPostRequestElement('network_type_id', bigintval(getRequestElement('network_type_id')));

	/*
	 * Some parameters are optional, at least one must be given so check a bunch
	 * of parameters.
	 */
	foreach (array('network_min_waiting_time', 'network_min_remain_clicks', 'network_min_payment', 'network_allow_erotic') as $element) {
		// Is this element empty?
		if (postRequestElement($element) == '') {
			// Then unset it to get a NULL for optional parameter
			unsetPostRequestElement($element);
		} // END - if
	} // END - foreach

	// Initialize variables
	$content = array();
	$id = 'network_max_reload_time_ye';
	$skip = false;

	// Get all POST data
	$postData = postRequestArray();

	// Convert "reload time selections"
	convertSelectionsToEpocheTime($postData, $content, $id, $skip);

	// Set the POST array back
	setPostRequestArray($postData);

	// Is there already an entry?
	if (isNetworkTypeHandlerConfigured(getRequestElement('network_id'), getRequestElement('network_type_id'))) {
		// This network type handler is already configured
		displayMessage('{--ADMIN_NETWORK_HANDLER_TYPE_HANDLER_ALREADY_CONFIGURED--}');
		return;
	} // END - if

	// Copy 'set all' and remove it from POST data
	$setAll = (postRequestElement('set_all') === 'Y');
	unsetPostRequestElement('set_all');

	// Shall we set for all?
	if ($setAll === true) {
		// Get all handlers
		$result = SQL_QUERY_ESC('SELECT `network_type_id` FROM `{?_MYSQL_PREFIX?}_network_types` WHERE `network_id`=%s ORDER BY `network_type_id` ASC',
			array(bigintval(getRequestElement('network_id'))), __FUNCTION__, __LINE__);

		// Do we have entries?
		if (SQL_HASZERONUMS($result)) {
			// No, then abort here
			displayMessage('{--ADMIN_CONFIG_NETWORK_HANDLER_SET_ALL_404--}');
			return;
		} // END - if

		// Init number of rows
		$numRows = 0;

		// Fetch all ids
		while (list($typeId) = SQL_FETCHROW($result)) {
			// Set it in GET data
			setGetRequestElement('network_type_id', $typeId);

			// Call this function again
			$numRows += doAdminNetworkProcessAddHandlerTypesConfig(false);
		} // END - while

		// Free result
		SQL_FREERESULT($result);

		// Output message
		if ($numRows > 0) {
			// Something has been updated
			displayMessage('{%message,ADMIN_CONFIG_NETWORK_HANDLER_TYPE_ALL_HANDLER_SAVED=' . bigintval($numRows) . '%}');
		} else {
			// Nothing has been saved
			loadTemplate('admin_settings_unsaved', false, '{--ADMIN_CONFIG_NETWORK_HANDLER_TYPE_HANDLER_NOT_CHANGED--}');
		}
	} else {
		// Get SQL query for new entry
		$SQL = getInsertSqlFromArray(postRequestArray(), 'network_types_config');

		// Run the query
		SQL_QUERY($SQL, __FUNCTION__, __LINE__);

		// Shall we display the message?
		if ($displayMessage === true) {
			// Output message
			if (!SQL_HASZEROAFFECTED()) {
				// Successfully added
				displayMessage('{--ADMIN_CONFIG_NETWORK_HANDLER_TYPE_HANDLER_SAVED--}');
			} else {
				// Not added
				loadTemplate('admin_settings_unsaved', false, '{--ADMIN_CONFIG_NETWORK_HANDLER_TYPE_HANDLER_NOT_SAVED--}');
			}
		} else {
			// Return amount of affected rows (1 or 2)
			return SQL_AFFECTEDROWS();
		}
	}
}

// Only changes network type configuration if not yet present
function doAdminNetworkProcessEditHandlerTypesConfig ($displayMessage = true) {
	// Remove the 'ok' part
	unsetPostRequestElement('ok');

	/*
	 * Some parameters are optional, at least one must be given so check a bunch
	 * of parameters.
	 */
	foreach (array('network_min_waiting_time', 'network_min_remain_clicks', 'network_min_payment', 'network_allow_erotic') as $element) {
		// Is this element empty?
		if (postRequestElement($element) == '') {
			// Then unset it to get a NULL for optional parameter
			unsetPostRequestElement($element);
		} // END - if
	} // END - foreach

	// Initialize variables
	$content = array();
	$id = 'network_max_reload_time_ye';
	$skip = false;
	$postData = postRequestArray();

	// Convert "reload time selections"
	convertSelectionsToEpocheTime($postData, $content, $id, $skip);

	// Set the POST array back
	setPostRequestArray($postData);

	// Is there already an entry?
	if (!isNetworkTypeHandlerConfigured(getRequestElement('network_id'), getRequestElement('network_type_id'))) {
		// This network type handler is not configured
		displayMessage('{--ADMIN_NETWORK_HANDLER_TYPE_HANDLER_NOT_CONFIGURED--}');
		return;
	} // END - if

	// Copy 'set all' and remove it from POST data
	$setAll = (postRequestElement('set_all') === 'Y');
	unsetPostRequestElement('set_all');

	// Shall we set for all?
	if ($setAll === true) {
		// Get all data entries
		$result = SQL_QUERY_ESC('SELECT `network_data_id` FROM `{?_MYSQL_PREFIX?}_network_types_config` WHERE `network_id`=%s ORDER BY `network_type_id` ASC',
			array(bigintval(getRequestElement('network_id'))), __FUNCTION__, __LINE__);

		// Do we have entries?
		if (SQL_HASZERONUMS($result)) {
			// No, then abort here
			displayMessage('{--ADMIN_CONFIG_NETWORK_HANDLER_SET_ALL_404--}');
			return;
		} // END - if

		// Init number of rows
		$numRows = 0;

		// Fetch all ids
		while (list($dataId) = SQL_FETCHROW($result)) {
			// Set it in GET data
			setPostRequestElement('network_data_id', $dataId);

			// Call this function again
			$numRows += doAdminNetworkProcessEditHandlerTypesConfig(false);
		} // END - while

		// Free result
		SQL_FREERESULT($result);

		// Output message
		if ($numRows > 0) {
			// Something has been updated
			displayMessage('{%message,ADMIN_CONFIG_NETWORK_HANDLER_TYPE_ALL_HANDLER_SAVED=' . bigintval($numRows) . '%}');
		} else {
			// Nothing has been saved
			loadTemplate('admin_settings_unsaved', false, '{--ADMIN_CONFIG_NETWORK_HANDLER_TYPE_HANDLER_NOT_CHANGED--}');
		}
	} else {
		// Get SQL query for new entry
		$SQL = getUpdateSqlFromArray(postRequestArray(), 'network_types_config', 'network_data_id', postRequestElement('network_data_id'), array('network_data_id'));

		// Run the query
		SQL_QUERY($SQL, __FUNCTION__, __LINE__);

		// Shall we display the message?
		if ($displayMessage === true) {
			// Output message
			if (!SQL_HASZEROAFFECTED()) {
				// Successfully added
				displayMessage('{--ADMIN_CONFIG_NETWORK_HANDLER_TYPE_HANDLER_SAVED--}');
			} else {
				// Not added
				loadTemplate('admin_settings_unsaved', false, '{--ADMIN_CONFIG_NETWORK_HANDLER_TYPE_HANDLER_NOT_CHANGED--}');
			}
		} else {
			// Return amount of affected rows (1 or 2)
			return SQL_AFFECTEDROWS();
		}
	}
}

// Do expression code for this extension
function doExpressionNetwork ($data) {
	// Construct replacer
	$replacer = sprintf(
		"{DQUOTE} . %s(%s, '%s') . {DQUOTE}",
		$data['callback'],
		$data['matches'][4][$data['key']],
		$data['extra_func']
	);

	// Replace %network_id% with the current network id
	$replacer = str_replace('%network_id%', getCurrentNetworkId(), $replacer);

	// Replace the code
	$code = replaceExpressionCode($data, $replacer);

	// Return it
	return $code;
}

// ----------------------------------------------------------------------------
//                     Call-back functions for exporting data
// ----------------------------------------------------------------------------

// Callback function to export network tables
function doAdminNetworkProcessExport () {
	// Init table with all valid what->table entries
	$validExports = array(
		// General network data
		'list_networks'                  => 'network_data',
		// Network type handler
		'list_network_types'             => 'network_types',
		// Network request parameter
		'list_network_request_params'    => 'network_request_params',
		// Network API response array index translation
		'list_network_array_translation' => 'network_array_translation',
	);

	// Is the 'what' key valid?
	if (!isset($validExports[getWhat()])) {
		// Not valid
		debug_report_bug(__FUNCTION__, __LINE__, 'what=' . getWhat() . ' - not supported');
	} // END - if

	// Generate call-back, some tables require to export not all columns
	$callbackName = 'doAdminNetworkExport' . capitalizeUnderscoreString($validExports[getWhat()]);

	// Is the call-back function there?
	if (!function_exists($callbackName)) {
		// No, this is really bad
		debug_report_bug(__FUNCTION__, __LINE__, 'Invalid call-back function ' . $callbackName . ' detected.');
	} elseif (isset($GLOBALS[__FUNCTION__][$callbackName])) {
		// Already called!
		debug_report_bug(__FUNCTION__, __LINE__, 'Double-call of export function ' . $callbackName . ' detected.');
	}

	// Call the function
	call_user_func($callbackName);

	// Mark it as called
	$GLOBALS[__FUNCTION__][$callbackName] = true;

	// Don't display the list/add new form
	$GLOBALS['network_display'] = false;
}

// Exports (and displays) the table 'network_data'
function doAdminNetworkExportNetworkData () {
	// Query for all networks
	$result = SQL_QUERY('SELECT
	`network_short_name`,
	`network_title`,
	`network_reflink`,
	`network_data_separator`,
	`network_row_separator`,
	`network_request_type`,
	`network_charset`,
	`network_require_id_card`,
	`network_query_amount`,
	`network_active`
FROM
	`{?_MYSQL_PREFIX?}_network_data`
ORDER BY
	`network_id` ASC',
		__FUNCTION__, __LINE__);

	// Start an empty SQL query
	$SQL = "INSERT INTO `&#123;&#63;_MYSQL_PREFIX&#63;&#125;_network_data` (`network_short_name`,`network_title`,`network_reflink`,`network_data_separator`,`network_row_separator`,`network_request_type`,`network_charset`,`network_require_id_card`,`network_query_amount`,`network_active`) VALUES\n";

	// Load all entries
	while ($content = SQL_FETCHARRAY($result)) {
		// Add row
		$SQL .= "('" .
			$content['network_short_name'] . "', '" .
			$content['network_title'] . "', '" .
			$content['network_reflink'] . "', '" .
			$content['network_data_separator'] . "', '" .
			$content['network_row_separator'] . "', '" .
			$content['network_request_type'] . "', '" .
			$content['network_charset'] . "', '" .
			$content['network_require_id_card'] . "', " .
			$content['network_query_amount'] . ", '" .
			$content['network_active'] . "'),\n";
	} // END - while

	// Remove last commata and close braces
	$SQL = substr($SQL, 0, -2);

	// Free result
	SQL_FREERESULT($result);

	// Output the SQL query
	loadTemplate('admin_export_network_data', false, $SQL);
}

// Exports (and displays) the table 'network_types'
function doAdminNetworkExportNetworkTypes () {
	// 'network_id' must be set
	if (!isGetRequestElementSet('network_id')) {
		// Only network handlers of one network will be exported per time
		debug_report_bug(__FUNCTION__, __LINE__, 'network_id not provided, please fix your links.');
	} // END - if

	// Get all network types of given network
	$result = SQL_QUERY_ESC('SELECT
	`network_type_id`,
	`network_id`,
	`network_type_handler`,
	`network_type_api_url`,
	`network_type_click_url`,
	`network_type_banner_url`,
	`network_type_reload_time_unit`
FROM
	`{?_MYSQL_PREFIX?}_network_types`
WHERE
	`network_id`=%s
ORDER BY
	`network_type_id` ASC',
		array(
			bigintval(getRequestElement('network_id'))
		), __FUNCTION__, __LINE__);

	// Start an empty SQL query
	$SQL = "INSERT INTO `&#123;&#63;_MYSQL_PREFIX&#63;&#125;_network_types` (`network_type_id`,`network_id`,`network_type_handler`,`network_type_api_url`,`network_type_click_url`,`network_type_banner_url`,`network_type_reload_time_unit`) VALUES\n";

	// Load all entries
	while ($content = SQL_FETCHARRAY($result)) {
		// Add row
		$SQL .= '(' .
			$content['network_type_id'] . ', ' .
			$content['network_id'] . ", '" .
			$content['network_type_handler'] . "', '" .
			$content['network_type_api_url'] . "', '" .
			$content['network_type_click_url'] . "', ";
		
		// Is the column NULL?
		if (is_null($content['network_type_banner_url'])) {
			// Column is NULL
			$SQL .= 'NULL';
		} else {
			// Column is set
			$SQL .= "'" . $content['network_type_banner_url'] . "'";
		}

		// Add more
		$SQL .= ",'" . $content['network_type_reload_time_unit'] . "'),\n";
	} // END - while

	// Remove last commata and close braces
	$SQL = substr($SQL, 0, -2);

	// Free result
	SQL_FREERESULT($result);

	// Output the SQL query
	loadTemplate('admin_export_network_types', false, $SQL);
}

// Exports (and displays) the table 'network_request_params'
function doAdminNetworkExportNetworkRequestParams () {
	// 'network_id' must be set
	if (!isGetRequestElementSet('network_id')) {
		// Only network request parameters of one network will be exported per time
		debug_report_bug(__FUNCTION__, __LINE__, 'network_id not provided, please fix your links.');
	} // END - if

	// Get all network types of given network
	$result = SQL_QUERY_ESC('SELECT
	`network_id`,
	`network_type_id`,
	`network_request_param_key`,
	`network_request_param_value`,
	`network_request_param_default`
FROM
	`{?_MYSQL_PREFIX?}_network_request_params`
WHERE
	`network_id`=%s
ORDER BY
	`network_type_id` ASC ,
	`network_request_param_id` ASC',
		array(
			bigintval(getRequestElement('network_id'))
		), __FUNCTION__, __LINE__);

	// Start an empty SQL query
	$SQL = "INSERT INTO `&#123;&#63;_MYSQL_PREFIX&#63;&#125;_network_request_params` (`network_id`,`network_type_id`,`network_request_param_key`,`network_request_param_value`,`network_request_param_default`) VALUES\n";

	// Load all entries
	while ($content = SQL_FETCHARRAY($result)) {
		// Add row
		$SQL .= '(' .
			$content['network_id'] . ', ' .
			$content['network_type_id'] . ", '" .
			$content['network_request_param_key'] . "', '" .
			$content['network_request_param_value'] . "', ";
		
		// Is the column NULL?
		if (is_null($content['network_request_param_default'])) {
			// Column is NULL
			$SQL .= "NULL),\n";
		} else {
			// Column is set
			$SQL .= "'" . $content['network_request_param_default'] . "'),\n";
		}
	} // END - while

	// Remove last commata and close braces
	$SQL = substr($SQL, 0, -2);

	// Free result
	SQL_FREERESULT($result);

	// Output the SQL query
	loadTemplate('admin_export_network_request_params', false, $SQL);
}

// Exports (and displays) the table 'network_array_translation'
function doAdminNetworkExportNetworkArrayTranslation () {
	// 'network_id' must be set
	if (!isGetRequestElementSet('network_id')) {
		// Only network API array index translations of one network will be exported per time
		debug_report_bug(__FUNCTION__, __LINE__, 'network_id not provided, please fix your links.');
	} // END - if

	// Get all network types of given network
	$result = SQL_QUERY_ESC('SELECT
	`network_id`,
	`network_type_id`,
	`network_array_index`,
	`sort`
FROM
	`{?_MYSQL_PREFIX?}_network_array_translation`
WHERE
	`network_id`=%s
ORDER BY
	`network_type_id` ASC,
	`sort` ASC',
		array(
			bigintval(getRequestElement('network_id'))
		), __FUNCTION__, __LINE__);

	// Start an empty SQL query
	$SQL = "INSERT INTO `&#123;&#63;_MYSQL_PREFIX&#63;&#125;_network_array_translation` (`network_id`,`network_type_id`,`network_array_index`,`sort`) VALUES\n";

	// Load all entries
	while ($content = SQL_FETCHARRAY($result)) {
		// Add row
		$SQL .= '(' .
			$content['network_id'] . ', ' .
			$content['network_type_id'] . ', ' .
			$content['network_array_index'] . ', ' .
			$content['sort'] . "),\n";
	} // END - while

	// Remove last commata and close braces
	$SQL = substr($SQL, 0, -2);

	// Free result
	SQL_FREERESULT($result);

	// Output the SQL query
	loadTemplate('admin_export_network_array_translation', false, $SQL);
}

// [EOF]
?>
